@@ -101,9 +101,9 @@ private:
 
 	void run(void);
 
-	ssize_t read(unsigned char *buffer, size_t len);
+	int read(unsigned char *buffer, size_t len);
 
-	void characters(const unsigned char *text, unsigned len);
+	void characters(const unsigned char *text, size_t len);
         void startElement(const unsigned char *name,
                 const unsigned char **attrib);
         void endElement(const unsigned char *name);
