@@ -67,6 +67,8 @@ BayonneBinder("ivrscript1")
 			(Check)&Checks::chkConnect},
 		{"join", false, (Method)&Methods::scrJoin,
 			(Check)&Checks::chkJoin},
+		{"reconnect", false, (Method)&Methods::scrReconnect,
+			(Check)&Checks::chkReconnect},
                 {NULL, false, NULL, NULL}}; 
 
 	char buf[256];
