@@ -31,11 +31,23 @@ const char *Checks::chkJoin(Line *line, ScriptImage *img)
 	return NULL;
 }
 
+const char *Checks::chkReconnect(Line *line, ScriptImage *img)
+{
+	unsigned idx = 0;
+	if(!useKeywords(line, "=encoding=framing"))
+		return "invalid keyword used for reconnect";
+
+	if(getOption(line, &idx))
+		return "no arguments for reconnect";
+
+	return NULL;
+}
+
 const char *Checks::chkConnect(Line *line, ScriptImage *img)
 {
 	unsigned idx = 0;
 	
-	if(!useKeywords(line, "=session=caller=display=timeout=duration=tone"))
+	if(!useKeywords(line, "=session=caller=display=timeout=duration=tone=encoding=framing"))
 		return "invalid keyword used for connect";
 
 	if(!getOption(line, &idx))
