@@ -35,6 +35,7 @@ static Keydata::Define driver[] = {
 	{"events", "128"},
 	{"priority", "0"},
 	{"encoding", "mulaw"},
+	{"peering", "any"},
 	{"framing", "20"},
 	{"timer", "500"},
 	{"payload", "0"},
@@ -51,7 +52,7 @@ static Keydata::Define driver[] = {
 	{"filler", "false"},
 	{"urlprefix", "sip:"},
 	{"immediate", "yes"},
-	{"peering", "friend"},	// default registration type
+	{"regtype", "friend"},	// default registration type
 	{"starting", "delayed"},
 	{"threads", "5"},
 	{"realm", "Bayonne"},
@@ -758,6 +759,33 @@ void Driver::startDriver(void)
 		snprintf(cbuf, sizeof(cbuf), "%d G726-32/%ld",
 			data_negotiate, info.rate);
 		break;
+	case g723_2bit:
+                if(!data_negotiate)
+                {
+                        data_negotiate = 94;
+                        setValue("payload", "94");
+                }
+                snprintf(cbuf, sizeof(cbuf), "%d G726-16/%ld",
+                        data_negotiate, info.rate);
+                break;
+	case g723_3bit:
+		if(!data_negotiate)
+		{
+			data_negotiate = 96;
+			setValue("payload", "96");
+		}
+		snprintf(cbuf, sizeof(cbuf), "%d G726-24/%ld",
+			data_negotiate, info.rate);
+		break;
+        case g723_5bit:
+                if(!data_negotiate)
+                {
+                        data_negotiate = 95;
+                        setValue("payload", "95");
+                }
+                snprintf(cbuf, sizeof(cbuf), "%d G726-40/%ld",
+                        data_negotiate, info.rate);
+                break;
 	case gsmVoice:
 		if(!data_negotiate)
 		{
@@ -1253,16 +1281,14 @@ const char *Driver::registerScript(ScriptImage *img, Line *line)
 	if(ext)
 	{
 		if(!ScriptChecks::useKeywords(line, 
-			"=peering=secret=encoding=framing=dtmf=user=count=device=info"))
+			"=reconnect=peering=secret=encoding=framing=dtmf=user=count=device=info"))
 				return "invalid keywords used for sip external";
 	}
 	else if(!ScriptChecks::useKeywords(line, 
-		"=uri=timeout=server=proxy=userid=secret=type=realm=dtmf"
+		"=uri=timeout=server=proxy=userid=secret=type=realm=dtmf=reconnect"
 		"=limit=public=encoding=framing=local=caller=service=peering"))
 			return "invalid keywords for sip registry";
 
-
-
 	reg->protocol = "sip";
 	reg->iface = getLast("interface");
 	reg->uri = ScriptChecks::findKeyword(line, "uri");
@@ -1276,9 +1302,7 @@ const char *Driver::registerScript(ScriptImage *img, Line *line)
 	reg->peering = ScriptChecks::findKeyword(line, "peering");
 	reg->encoding = ScriptChecks::findKeyword(line, "encoding");
 	reg->caller = ScriptChecks::findKeyword(line, "caller");
-
-	if(reg->peering)
-		reg->encoding = reg->peering;
+	reg->reconnect = ScriptChecks::findKeyword(line, "reconnect");
 
 	if(ext)
 		reg->localid = ScriptChecks::findKeyword(line, "user");
@@ -1358,12 +1382,28 @@ const char *Driver::registerScript(ScriptImage *img, Line *line)
 			if(cp)
 				reg->framing = atol(cp);
 		}
+		cp = cfg->getLast("reconnect");
+		if(cp && !reg->reconnect)
+			reg->reconnect = ip->dupString(cp);
 		cp = cfg->getLast("peering");
-		if(cp)
-			reg->encoding = reg->peering = ip->dupString(cp);
+		if(cp && !strnicmp(cp, "no", 2))
+			reg->peering = NULL;
+		else if(cp && !reg->peering)
+			reg->peering = ip->dupString(cp);
 		delete cfg;
 	}
 #endif
+	if(reg->peering)
+	{
+		if(!stricmp(reg->peering, "yes"))
+			reg->peering = getLast("peering");
+
+		if(!reg->peering)
+			reg->peering = "any";
+
+		if(!strnicmp(reg->peering, "no", 2))
+			reg->peering = NULL;
+	}
 
 	while(insecure && *insecure)
 	{
@@ -1396,7 +1436,7 @@ const char *Driver::registerScript(ScriptImage *img, Line *line)
 	}
 
 	if(!reg->type)
-		reg->type = getLast("peering");
+		reg->type = getLast("regtype");
 
 	if(!reg->hostid || !*reg->hostid)
 		reg->hostid = server->getLast("server");
@@ -1667,7 +1707,7 @@ void Driver::run(void)
 	sdp_message_t *remote_sdp, *local_sdp;
 	sdp_connection_t *conn;
 	osip_content_type_t *ct;
-	sdp_media_t *remote_media, *local_media, *audio_media;
+	sdp_media_t *remote_media = NULL, *local_media, *audio_media;
 	char *tmp, *audio_payload, *adp;
 	const char *data_rtpmap = NULL;
 	int pos, mpos, rtn;
@@ -1931,45 +1971,11 @@ exitcall:
 			local_sdp = NULL;
 			remote_sdp = NULL;
 			conn = NULL;
-			eXosip_lock();
-			eXosip_call_build_ack(sevent->did, &msg);
 			if(sevent->request && sevent->response)
 			{
 				local_sdp = eXosip_get_sdp_info(sevent->request);
 				remote_sdp = eXosip_get_sdp_info(sevent->response);
 			}
-			eXosip_unlock();
-			data_rtpmap = session->getSymbol("session.sdp_encoding");
-                        cp = session->getSymbol("session.ip_public");
-			if(local_sdp && remote_sdp && session->dtmf_payload)
-                        	snprintf(buf, sizeof(buf),
-                                	"v=0\r\n"
-                                	"o=bayonne 0 0 IN IP4 %s\r\n"
-                                	"s=call\r\n"
-                                	"c=IN IP4 %s\r\n"
-                                	"t=0 0\r\n"
-                                	"m=audio %d RTP/AVP %d %d\r\n"
-                                	"%s\r\n"
-                                	"a=rtpmap:%d telephone-events/8000\r\n",
-                                	cp, cp,
-                                	session->getLocalPort(), 
-					session->data_payload, 
-					session->dtmf_payload,
-                                	data_rtpmap, 
-					session->dtmf_payload);
-			else if(local_sdp && remote_sdp)
-                        	snprintf(buf, sizeof(buf),
-                                	"v=0\r\n"
-                                	"o=bayonne 0 0 IN IP4 %s\r\n"
-                                	"s=call\r\n"
-                                	"c=IN IP4 %s\r\n"
-                                	"t=0 0\r\n"
-                                	"m=audio %d RTP/AVP %d\r\n"
-                                	"%s\r\n",
-                                	cp, cp,
-                                	session->getLocalPort(), 
-					session->data_payload, 
-                                	data_rtpmap);
 
 			// we should verify sdp stuff...
 
@@ -1978,7 +1984,10 @@ exitcall:
 				conn = eXosip_get_audio_connection(remote_sdp);
 				remote_media = eXosip_get_audio_media(remote_sdp);
 			}
-			if(conn && remote_media)
+			cp = session->getSymbol("session.rtp_remote");
+			if(cp && !*cp)
+				cp = NULL;
+			if(conn && remote_media && !cp)
 			{
 				session->remote_address = conn->c_addr;
 				session->remote_port = atoi(remote_media->m_port);
@@ -1992,19 +2001,14 @@ exitcall:
 				session->setConst("session.rtp_local", cbuf);
 			}
 
-			eXosip_lock();
-			osip_message_set_body(msg, buf, strlen(buf));
-			osip_message_set_content_type(msg, "application/sdp");
-			eXosip_unlock();
-
                         memset(&event, 0, sizeof(event));
                         event.id = CALL_ANSWERED;
                         if(session->postEvent(&event))
 			{
 				eXosip_lock();
+				eXosip_call_build_ack(sevent->did, &msg);
 				rtn = eXosip_call_send_ack(sevent->did, msg);
 				eXosip_unlock();
-				msg = NULL;	
 				memset(&event, 0, sizeof(event));
 				if(rtn)
 					event.id = CALL_FAILURE;
@@ -2015,8 +2019,6 @@ exitcall:
 			if(sdp)
 				delString(sdp);
 
-			if(msg)
-				osip_message_free(msg);
 			if(local_sdp)
 				sdp_message_free(local_sdp);
 			if(remote_sdp)
@@ -2186,39 +2188,7 @@ killit:
 				"t=0 0\r\n", cp, cp);
 			
 			mpos = 0;
-			switch(session->info.encoding)
-			{
-			case mulawAudio:
-				data_rtpmap = "PCMU/8000/1";
-				break;
-			case alawAudio:
-				data_rtpmap = "PCMA/8000/1";
-				break;
-			case g721ADPCM:
-				data_rtpmap = "G726-32/8000/1";
-				break;
-			case speexVoice:
-				data_rtpmap = "SPEEX/8000/1";
-				break;
-			case gsmVoice:
-				data_rtpmap = "GSM/8000/1";
-				break;
-			case pcm16Stereo:
-				data_rtpmap = "L16/11025/2";
-				break;
-			case pcm16Mono:
-				switch(session->info.rate)
-				{
-				case 16000:
-					data_rtpmap = "L16/16000/1";
-					break;
-				default:
-					data_rtpmap = "L16/8000/1";
-					break;
-				}
-			default:
-				break;
-			}
+			data_rtpmap = session->sdpEncoding();
 			audio_payload = NULL;
 			remote_sdp = eXosip_get_sdp_info(sevent->request);
 			if(!remote_sdp)
@@ -2737,48 +2707,8 @@ sip415:
 			session->dtmf_payload = 0;
 
 			mpos = 0;
-			switch(session->info.encoding)
-			{
-			case mulawAudio:
-				session->data_payload = 0;
-				data_rtpmap = "PCMU/8000/1";
-				break;
-			case alawAudio:
-				session->data_payload = 8;
-				data_rtpmap = "PCMA/8000/1";
-				break;
-			case g721ADPCM:
-				session->data_payload = 5;
-				data_rtpmap = "G726-32/8000/1";
-				break;
-			case speexVoice:
-				session->data_payload = 97;
-				data_rtpmap = "SPEEX/8000/1";
-				break;
-			case gsmVoice:
-				session->data_payload = 3;
-				data_rtpmap = "GSM/8000/1";
-				break;
-			case pcm16Stereo:
-				session->data_payload = 98;
-				data_rtpmap = "L16/11025/2";
-				session->info.rate = 11025;
-				break;
-			case pcm16Mono:
-				switch(session->info.rate)
-				{
-				case 16000:
-					session->data_payload = 98;
-					data_rtpmap = "L16/16000/1";
-					break;
-				default:
-					session->info.rate = 8000;
-					session->data_payload = 97;
-					data_rtpmap = "L16/8000/1";
-				}
-			default:
-				break;
-			}
+			data_rtpmap = session->sdpEncoding();
+			session->data_payload = session->sdpPayload();
 			audio_payload = NULL;
 			while(!osip_list_eol(remote_sdp->m_medias, mpos))
 			{
@@ -2860,7 +2790,9 @@ sip415:
 			data_rtpmap = session->getSymbol("session.sdp_encoding");
 
 			if(reg && reg->peering)
-				session->setConst("session.sdp_peering", data_rtpmap);
+				session->setPeering(reg->peering);
+			if(reg && reg->reconnect)
+				session->setConst("session.sdp_reconnect", reg->reconnect);
 
 			if(session->dtmf_payload)
 				snprintf(cbuf, sizeof(cbuf),
