@@ -127,6 +127,7 @@ public:
 	const char *dtmf;
 	const char *encoding;
 	const char *peering;
+	const char *reconnect;
 	const char *caller;
 	timeout_t framing;
 	int regid;
@@ -300,7 +301,7 @@ protected:
 	AudioCodec *peer_codec;
 	uint8 dtmf_payload, data_payload;
 	bool dtmf_sipinfo;
-	bool reinvite;
+	bool reinvite, reconnecting;
 	volatile bool dtmf_inband;
 	volatile time_t session_timer;
 
@@ -324,6 +325,10 @@ public:
 	const char *audioExtension(void);
 	const char *checkAudio(bool live);
 
+        uint8 sdpPayload();
+        const char *sdpEncoding();
+	void setPeering(const char *reg);
+
 	// core timer virtuals all port session objects need to define
 	timeout_t getRemaining(void);
 	void startTimer(timeout_t timer);
@@ -333,6 +338,8 @@ public:
 	
 	void stopRTP(void);
 
+	void suspendRTP(void);
+
 	void makeIdle(void);
 
 	void clrAudio(void);
@@ -353,6 +360,8 @@ public:
 
 	bool enterTone(Event *event);
 
+	bool enterReconnect(Event *event);
+
 	bool enterXfer(Event *event);
 
 	timeout_t getToneFraming(void);
