@@ -40,6 +40,7 @@ TimerPort()
 	dtmf_sipinfo = false;
 
 	rtp_flag = update_pos = false;
+	reconnecting = false;
 
 	peer_buffer = NULL;
 	peer_codec = NULL;
@@ -51,6 +52,94 @@ Session::~Session()
 	stopRTP();
 }
 
+uint8 Session::sdpPayload(void)
+{
+	switch(info.encoding)
+	{
+	case Audio::mulawAudio:
+		return 0;
+	case Audio::alawAudio:
+		return 8;
+	case Audio::speexAudio:
+	case Audio::speexVoice:
+		return 97;
+	case Audio::g721ADPCM:
+		return 5;
+	case Audio::g723_3bit:
+		return 96;
+	case Audio::g723_5bit:
+		return 95;
+	case Audio::g723_2bit:
+		return 94;
+	case Audio::gsmVoice:
+		return 3;
+	default:
+		return 90;
+	}
+}
+
+const char *Session::sdpEncoding(void)
+{
+	switch(info.encoding)
+	{
+	case Audio::gsmVoice:
+		return "GSM/8000/1";
+	case Audio::mulawAudio:
+		return "PCMU/8000/1";
+	case Audio::alawAudio:
+		return "PCMA/8000/1";
+	case Audio::pcm16Mono:
+		return "L16/11025/1";
+	case Audio::pcm16Stereo:
+		return "L16/11025/2";
+	case Audio::g721ADPCM:
+		return "G726-32/8000/1";
+	case Audio::g723_2bit:
+		return "G726-16/8000/1";
+	case Audio::g723_3bit:
+		return "G726-24/8000/1";
+	case Audio::g723_5bit:
+		return "G726-40/8000/1";
+	case Audio::g729Audio:
+		return "G729/8000/1";	
+	case Audio::speexVoice:
+		return "SPEEX/8000/1";
+	case Audio::speexAudio:
+		return "SPEEX/16000/1";
+	default:
+		return NULL;
+	}
+}
+
+void Session::setPeering(const char *enc)
+{
+	if(!enc)
+		return;
+
+	if(!stricmp(enc, "gsm"))
+		enc = "a=rtpmap:3 GSM/8000/1";
+	else if(!stricmp(enc, "ulaw") || !stricmp(enc, "mulaw"))
+		enc = "a=rtpmap:0 PCMU/8000/1";
+	else if(!stricmp(enc, "alaw"))
+		enc = "a=rtpmap:8 PCMU/8000/1";
+	else if(!stricmp(enc, "adpcm"))
+		enc = "a=rtpmap:97 G726-32/8000/1";
+	else if(!stricmp(enc, "stereo"))
+		enc = "a=rtpmap:98 L16/11025/2";
+	else if(!stricmp(enc, "g726-16"))
+		enc = "a=rtpmap:94 G726-16/8000/1";
+	else if(!stricmp(enc, "g726-24"))
+		enc = "a=rtpmap:95 G726-24/8000/1";
+	else if(!stricmp(enc, "g726-40"))
+		enc = "a=rtpmap:96 G726-40/8000/1";
+	else if(!stricmp(enc, "g729"))
+		enc = "a=rtpmap:18 G729/8000/1";
+	else
+		enc = getSymbol("session.sdp_encoding");
+
+	setConst("session.sdp_peering", enc);
+}
+
 void Session::redirect(const char *target, const char *encoding, unsigned char data, unsigned char dtmf)
 {
 	osip_message_t *msg = NULL;
@@ -110,16 +199,11 @@ void Session::redirect(const char *target, const char *encoding, unsigned char d
 void Session::reconnect(void)
 {
 	const char *target = getSymbol("session.rtp_local");
-	const char *encoding = getSymbol("session.sdp_encoding");
 	
-	if(!encoding || !target || !reinvite)
+	if(!target || !reinvite)
 		return;
 
-	encoding = strchr(encoding, ' ');
-	if(!encoding)
-		return;
-	++encoding;
-	redirect(target, encoding, data_payload, dtmf_payload);
+	redirect(target, sdpEncoding(), data_payload, dtmf_payload);
 	reinvite = false;
 }
 
@@ -215,10 +299,31 @@ void Session::setEncoding(const char *encoding, timeout_t framing)
 		AudioCodec::endCodec(codec);
 
 	memset(&info, 0, sizeof(info));
-	info.encoding = Audio::getEncoding(encoding);
+
+	if(!stricmp(encoding, "stereo"))
+		info.encoding = Audio::pcm16Stereo;
+	else if(!stricmp(encoding, "g726-40"))
+		info.encoding = Audio::g723_5bit;
+	else if(!stricmp(encoding, "g726-24"))
+		info.encoding = Audio::g723_3bit;
+	else if(!stricmp(encoding, "g726-16"))
+		info.encoding = Audio::g723_2bit;
+	else
+		info.encoding = Audio::getEncoding(encoding);
 
 	switch(info.encoding)
 	{
+	case Audio::pcm16Stereo:
+	case Audio::pcm16Mono:
+		info.rate = (Audio::Rate)11025;
+		if(!framing)
+			framing = 20;
+		break;
+	case Audio::speexAudio:
+		info.rate = (Audio::Rate)16000;
+		if(!framing)
+			framing = 20;
+		break;
 	default:
 		info.rate = Audio::rate8khz;
 		if(!framing)
@@ -295,6 +400,17 @@ void Session::startRTP(void)
 	rtp_flag = true;
 }		
 
+void Session::suspendRTP(void)
+{
+	if(!rtp_flag || !rtp)
+		return;
+
+	rtp_flag = false;
+
+	Thread::yield();
+	rtp->forgetDestination(remote_address, remote_port);
+}
+
 void Session::stopRTP(void)
 {
 	if(!rtp_flag || !rtp)
@@ -319,6 +435,7 @@ void Session::makeIdle(void)
 {
 	update_pos = false;
 	session_timer = 0;
+	reconnecting = false;
 
 	if(offhook)
 		sipRelease();
@@ -791,7 +908,9 @@ failure:
 
 		setConst("session.sdp_encoding", rtpmap);
 		if(reg && reg->peering)
-			setConst("session.sdp_peering", rtpmap);
+			setPeering(reg->peering);
+		if(reg && reg->reconnect)
+			setConst("session.sdp_reconnect", reg->reconnect);
 
 		if(dtmf_sipinfo)
 			setConst("session.dtmfmode", "info");
@@ -940,6 +1059,54 @@ bool Session::enterPickup(Event *event)
 	return false;
 }
 
+bool Session::enterReconnect(Event *event)
+{
+	const char *sdp;
+
+	switch(event->id)
+	{
+	case ENTER_RECONNECT:
+		sdp = getSymbol("session.sdp_reconnect");
+		if(!sdp || !*sdp)
+			return false;
+
+		if(!stristr(sdp, event->reconnect.encoding))
+			return false;
+
+		if(reconnecting)
+		{
+			reconnecting = false;
+			return false;
+		}
+		state.peering = false;
+		reinvite = true;
+		suspendRTP();
+		setEncoding(event->reconnect.encoding, event->reconnect.framing);
+		data_payload = sdpPayload();
+		state.timeout = 0;
+		return true;
+	case CALL_ANSWERED:
+		reconnecting = true;
+		startRTP();
+		return true;
+	case STREAM_ACTIVE:
+		setRunning();
+		return true;
+	case STREAM_PASSIVE:
+		reconnect();
+		return true;
+	case EXIT_RECONNECT:
+		if(isDisconnecting() && offhook)
+			sipHangup();
+		else if(!rtp_flag)
+			startRTP();
+		return true;
+	default:
+		return false;
+	}
+}
+
+
 bool Session::enterTone(Event *event)
 {
 	if(event->id == ENTER_STATE && audio.tone)
